Allow specifying position in the output file in bytes instead of
obs-sized blocks, improve compatibility with GNU dd.

function                                             old     new   delta
dd_main                                             1448    1518     +70
.rodata                                            26143   26167     +24
static.oflag_words                                     -      12     +12
------------------------------------------------------------------------------
(add/remove: 1/0 grow/shrink: 2/0 up/down: 106/0)             Total: 106 bytes

Signed-off-by: Rostislav Skudnov <rostis...@tuxera.com>
---
 coreutils/dd.c             | 33 ++++++++++++++++++++++++---------
 docs/posix_conformance.txt |  1 +
 2 files changed, 25 insertions(+), 9 deletions(-)

diff --git a/docs/posix_conformance.txt b/docs/posix_conformance.txt
index cdf89b7..0e6f4a3 100644
--- a/docs/posix_conformance.txt
+++ b/docs/posix_conformance.txt
@@ -182,6 +182,7 @@ dd compatibility options:
   conv=fsync      |  yes   |           |
   iflag=skip_bytes|  yes   |           |
   iflag=fullblock |  yes   |           |
+  oflag=seek_bytes|  yes   |           |
 
 df POSIX options
  option           | exists | compliant | remarks
diff --git a/coreutils/dd.c b/coreutils/dd.c
index 0f130bc..129637e 100644
--- a/coreutils/dd.c
+++ b/coreutils/dd.c
@@ -37,7 +37,7 @@
 //config:      elapsed time and speed.
 //config:
 //config:config FEATURE_DD_IBS_OBS
 //config:      bool "Enable ibs, obs, iflag and conv options"
 //config:      default y
 //config:      depends on DD
 //config:      help
@@ -57,7 +57,7 @@
 
 //usage:#define dd_trivial_usage
 //usage:       "[if=FILE] [of=FILE] " IF_FEATURE_DD_IBS_OBS("[ibs=N] [obs=N] 
") "[bs=N] [count=N] [skip=N]\n"
-//usage:       "       [seek=N]" IF_FEATURE_DD_IBS_OBS(" 
[conv=notrunc|noerror|sync|fsync] [iflag=skip_bytes|fullblock]")
+//usage:       "       [seek=N]" IF_FEATURE_DD_IBS_OBS(" 
[conv=notrunc|noerror|sync|fsync] [iflag=skip_bytes|fullblock] 
[oflag=seek_bytes]")
 //usage:#define dd_full_usage "\n\n"
 //usage:       "Copy a file with converting and formatting\n"
 //usage:     "\n       if=FILE         Read from FILE instead of stdin"
@@ -80,6 +80,7 @@
 //usage:     "\n       conv=swab       Swap every pair of bytes"
 //usage:     "\n       iflag=skip_bytes        skip=N is in bytes"
 //usage:     "\n       iflag=fullblock Read full blocks"
+//usage:     "\n       oflag=seek_bytes        seek=N is in bytes"
 //usage:       )
 //usage:       IF_FEATURE_DD_STATUS(
 //usage:     "\n       status=noxfer   Suppress rate output"
@@ -133,10 +134,14 @@ enum {
        FLAG_SKIP_BYTES = (1 << 5) * ENABLE_FEATURE_DD_IBS_OBS,
        FLAG_FULLBLOCK = (1 << 6) * ENABLE_FEATURE_DD_IBS_OBS,
        /* end of input flags */
-       FLAG_TWOBUFS = (1 << 7) * ENABLE_FEATURE_DD_IBS_OBS,
-       FLAG_COUNT   = 1 << 8,
-       FLAG_STATUS_NONE = 1 << 9,
-       FLAG_STATUS_NOXFER = 1 << 10,
+       /* start of output flags */
+       FLAG_OFLAG_SHIFT = 7,
+       FLAG_SEEK_BYTES = (1 << 7) * ENABLE_FEATURE_DD_IBS_OBS,
+       /* end of output flags */
+       FLAG_TWOBUFS = (1 << 8) * ENABLE_FEATURE_DD_IBS_OBS,
+       FLAG_COUNT   = 1 << 9,
+       FLAG_STATUS_NONE = 1 << 10,
+       FLAG_STATUS_NOXFER = 1 << 11,
 };
 
 static void dd_output_status(int UNUSED_PARAM cur_signal)
@@ -250,7 +255,7 @@ int dd_main(int argc UNUSED_PARAM, char **argv)
        static const char keywords[] ALIGN1 =
                
"bs\0""count\0""seek\0""skip\0""if\0""of\0"IF_FEATURE_DD_STATUS("status\0")
 #if ENABLE_FEATURE_DD_IBS_OBS
-               "ibs\0""obs\0""conv\0""iflag\0"
+               "ibs\0""obs\0""conv\0""iflag\0""oflag\0"
 #endif
                ;
 #if ENABLE_FEATURE_DD_IBS_OBS
@@ -258,6 +263,8 @@ int dd_main(int argc UNUSED_PARAM, char **argv)
                "notrunc\0""sync\0""noerror\0""fsync\0""swab\0";
        static const char iflag_words[] ALIGN1 =
                "skip_bytes\0""fullblock\0";
+       static const char oflag_words[] ALIGN1 =
+               "seek_bytes\0";
 #endif
 #if ENABLE_FEATURE_DD_STATUS
        static const char status_words[] ALIGN1 =
@@ -276,6 +283,7 @@ int dd_main(int argc UNUSED_PARAM, char **argv)
                OP_obs,
                OP_conv,
                OP_iflag,
+               OP_oflag,
                /* Must be in the same order as FLAG_XXX! */
                OP_conv_notrunc = 0,
                OP_conv_sync,
@@ -297,6 +305,7 @@ int dd_main(int argc UNUSED_PARAM, char **argv)
        //swab          swap every pair of input bytes: will abort on non-even 
reads
                OP_iflag_skip_bytes,
                OP_iflag_fullblock,
+               OP_oflag_seek_bytes,
 #endif
        };
        smallint exitcode = EXIT_FAILURE;
@@ -368,6 +377,10 @@ int dd_main(int argc UNUSED_PARAM, char **argv)
                        G.flags |= parse_comma_flags(val, iflag_words, "iflag") 
<< FLAG_IFLAG_SHIFT;
                        /*continue;*/
                }
+               if (what == OP_oflag) {
+                       G.flags |= parse_comma_flags(val, oflag_words, "oflag") 
<< FLAG_OFLAG_SHIFT;
+                       /*continue;*/
+               }
 #endif
                if (what == OP_bs) {
                        ibs = xatoul_range_sfx(val, 1, ((size_t)-1L)/2, 
cwbkMG_suffixes);
@@ -439,7 +452,8 @@ int dd_main(int argc UNUSED_PARAM, char **argv)
                xmove_fd(xopen(outfile, oflag), ofd);
 
                if (seek && !(G.flags & FLAG_NOTRUNC)) {
-                       if (ftruncate(ofd, seek * obs) < 0) {
+                       size_t blocksz = (G.flags & FLAG_SEEK_BYTES) ? 1 : obs;
+                       if (ftruncate(ofd, seek * blocksz) < 0) {
                                struct stat st;
 
                                if (fstat(ofd, &st) < 0
@@ -472,7 +486,8 @@ int dd_main(int argc UNUSED_PARAM, char **argv)
                }
        }
        if (seek) {
-               if (lseek(ofd, seek * obs, SEEK_CUR) < 0)
+               size_t blocksz = (G.flags & FLAG_SEEK_BYTES) ? 1 : obs;
+               if (lseek(ofd, seek * blocksz, SEEK_CUR) < 0)
                        goto die_outfile;
        }
 
@@ -37,7 +37,7 @@
 //config:
 //config:config FEATURE_DD_IBS_OBS
-//config:      bool "Enable ibs, obs, iflag and conv options"
+//config:      bool "Enable ibs, obs, iflag, oflag and conv options"
 //config:      default y
 //config:      depends on DD
-- 
2.1.4

_______________________________________________
busybox mailing list
busybox@busybox.net
http://lists.busybox.net/mailman/listinfo/busybox

Reply via email to